@@ -1,5 +1,11 @@
 <?php
+/**
+ * @package IMP
+ *
+ * $Horde: imp/lib/Crypt/PGP.php,v 1.90.2.26 2009/05/09 12:31:20 jan Exp $
+ */
 
+/** Horde_Crypt_pgp */
 require_once 'Horde/Crypt/pgp.php';
 
 /**
@@ -11,15 +17,12 @@ define('IMP_PGP_PUBKEY_FIELD', 'pgpPublicKey');
  * The IMP_PGP:: class contains all functions related to handling
  * PGP messages within IMP.
  *
- * $Horde: imp/lib/Crypt/PGP.php,v 1.90.2.19 2007/03/09 11:42:18 jan Exp $
- *
- * Copyright 2002-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @since   IMP 4.0
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @package IMP
  */
 class IMP_PGP extends Horde_Crypt_pgp {
@@ -29,21 +32,13 @@ class IMP_PGP extends Horde_Crypt_pgp {
      *
      * @var array
      */
-    var $_sources = array();
+    var $_sources = null;
 
     /**
      * Constructor
      */
     function IMP_PGP()
     {
-        /* Get the listing of all sources we search for public keys. */
-        if (($sources = $GLOBALS['prefs']->getValue('search_sources'))) {
-            $this->_sources = explode("\t", $sources);
-            if ((count($this->_sources) == 1) && empty($this->_sources[0])) {
-                $this->_sources = array();
-            }
-        }
-
         parent::Horde_Crypt_pgp(array('program' => $GLOBALS['conf']['utils']['gnupg'],
                                       'temp' => Horde::getTempDir()));
     }
@@ -51,19 +46,21 @@ class IMP_PGP extends Horde_Crypt_pgp {
     /**
      * Generate the personal Public/Private keypair and store in prefs.
      *
-     * @param string $realname            See Horde_Crypt_pgp::
-     * @param string $email               See Horde_Crypt_pgp::
-     * @param string $passphrase          See Horde_Crypt_pgp::
-     * @param string $comment             See Horde_Crypt_pgp::
-     * @param string $keylength           See Horde_Crypt_pgp::
+     * @param string $realname    See Horde_Crypt_pgp::
+     * @param string $email       See Horde_Crypt_pgp::
+     * @param string $passphrase  See Horde_Crypt_pgp::
+     * @param string $comment     See Horde_Crypt_pgp::
+     * @param string $keylength   See Horde_Crypt_pgp::
      *
-     * @return PEAR_Error  Returns a PEAR_Error object on error.
+     * @return PEAR_Error  Returns PEAR_Error object on error.
      */
     function generatePersonalKeys($name, $email, $passphrase, $comment = '',
                                   $keylength = 1024)
     {
         $keys = $this->generateKey($name, $email, $passphrase, $comment, $keylength);
-        if (is_a($keys, 'PEAR_Error')) return $keys;
+        if (is_a($keys, 'PEAR_Error')) {
+            return $keys;
+        }
 
         /* Store the keys in the user's preferences. */
         $this->addPersonalPublicKey($keys['public']);
@@ -145,9 +142,9 @@ class IMP_PGP extends Horde_Crypt_pgp {
         /* Store all signatures that appear in the key. */
         foreach ($key_info['signature'] as $id => $sig) {
             /* Check to make sure the key does not already exist in ANY
-               address book and removes the id from the key_info for a
-               correct output. */
-            $result = $this->getPublicKey($sig['email'], null, true);
+             * address book and remove the id from the key_info for a correct
+             * output. */
+            $result = $this->getPublicKey($sig['email'], null, false);
             if (!is_a($result, 'PEAR_Error') && !empty($result)) {
                 unset($key_info['signature'][$id]);
                 continue;
@@ -165,44 +162,77 @@ class IMP_PGP extends Horde_Crypt_pgp {
 
     /**
      * Retrieves a public key by e-mail.
-     * First, the key will be attempted to be retrieved from a user's
-     * address book(s).
-     * Second, if unsuccessful, the key is attempted to be retrieved via
-     * a public PGP keyserver.
+     *
+     * First, the key will be attempted to be retrieved from a user's address
+     * book(s).
+     * Second, if unsuccessful, the key is attempted to be retrieved via a
+     * public PGP keyserver.
      *
      * @param string $address      The e-mail address to search by.
      * @param string $fingerprint  The fingerprint of the user's key.
+     * @param boolean $server      Whether to check the publick key servers for
+     *                             the key.
      *
      * @return string  The PGP public key requested. Returns PEAR_Error object
      *                 on error.
      */
-    function getPublicKey($address, $fingerprint = null)
+    function getPublicKey($address, $fingerprint = null, $server = true)
     {
+        $result = false;
+
+        /* If there is a cache driver configured, try to get the public key
+         * from the cache. */
+        if (!empty($GLOBALS['conf']['cache']['driver'])) {
+            $cache = &Horde_Cache::singleton($GLOBALS['conf']['cache']['driver'], Horde::getDriverConfig('cache', $GLOBALS['conf']['cache']['driver']));
+            if (is_a($cache, 'PEAR_Error')) {
+                Horde::fatal($cache, __FILE__, __LINE__);
+            } else {
+                $result = $cache->get("PGPpublicKey_" . $address . $fingerprint, 3600);
+            }
+        }
+
+        if ($result) {
+            Horde::logMessage('PGPpublicKey: ' . serialize($result), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+            return $result;
+        }
+
         /* Try retrieving by e-mail only first. */
+        $this->_getPublicKeySources();
         $result = $GLOBALS['registry']->call('contacts/getField', array($address, IMP_PGP_PUBKEY_FIELD, $this->_sources, false, true));
 
         /* TODO: Retrieve by ID. */
 
-        /* Try retrieving via a PGP public keyserver. */
-        if (is_a($result, 'PEAR_Error') && !empty($fingerprint)) {
-            $result = $this->getFromPublicKeyserver($fingerprint);
-        }
-
         /* See if the address points to the user's public key. */
         if (is_a($result, 'PEAR_Error')) {
             require_once 'Horde/Identity.php';
             $identity = &Identity::singleton(array('imp', 'imp'));
             $personal_pubkey = $this->getPersonalPublicKey();
             if (!empty($personal_pubkey) && $identity->hasAddress($address)) {
-                return $personal_pubkey;
+                $result = $personal_pubkey;
             }
         }
 
+        /* Try retrieving via a PGP public keyserver. */
+        if ($server && is_a($result, 'PEAR_Error')) {
+            $result = $this->getFromPublicKeyserver($fingerprint, $address);
+
+            /* If there is a cache driver configured and a cache object exists,
+             * store the public key in the cache. */
+            if (is_object($cache) && !is_a($result, 'PEAR_Error')) {
+                $cache->set("PGPpublicKey_" . $address . $fingerprint, $result, 3600);
+            }
+        }
+
+        /* Return now, if no public key found at all. */
+        if (is_a($result, 'PEAR_Error')) {
+            return $result;
+        }
+
         /* If more than one public key is returned, just return the first in
          * the array. There is no way of knowing which is the "preferred" key,
          * if the keys are different. */
         if (is_array($result)) {
-            return reset($result);
+            reset($result);
         }
 
         return $result;
@@ -213,15 +243,11 @@ class IMP_PGP extends Horde_Crypt_pgp {
      *
      * @return array  All PGP public keys available. Returns PEAR_Error object
      *                on error.
-     *
      */
     function listPublicKeys()
     {
-        if (empty($this->_sources)) {
-            return array();
-        } else {
-            return $GLOBALS['registry']->call('contacts/getAllAttributeValues', array(IMP_PGP_PUBKEY_FIELD, $this->_sources));
-        }
+        $this->_getPublicKeySources();
+        return (empty($this->_sources)) ? array() : $GLOBALS['registry']->call('contacts/getAllAttributeValues', array(IMP_PGP_PUBKEY_FIELD, $this->_sources));
     }
 
     /**
@@ -233,9 +259,29 @@ class IMP_PGP extends Horde_Crypt_pgp {
      */
     function deletePublicKey($email)
     {
+        $this->_getPublicKeySources();
         return $GLOBALS['registry']->call('contacts/deleteField', array($email, IMP_PGP_PUBKEY_FIELD, $this->_sources));
     }
 
+    /**
+     * Obtains a list of sources for public keys.
+     *
+     * @access private
+     */
+    function _getPublicKeySources()
+    {
+        /* Get the listing of all sources we search for public keys. */
+        if ($this->_sources === null) {
+            $this->_sources = array();
+            if (($sources = $GLOBALS['prefs']->getValue('search_sources'))) {
+                $this->_sources = explode("\t", $sources);
+                if ((count($this->_sources) == 1) && empty($this->_sources[0])) {
+                    $this->_sources = array();
+                }
+            }
+        }
+    }
+
     /**
      * Parse a message into its PGP components.
      *
@@ -260,7 +306,7 @@ class IMP_PGP extends Horde_Crypt_pgp {
         reset($result);
         do {
             $block = current($result);
-            $temp_part = &new MIME_Part();
+            $temp_part = new MIME_Part();
             $temp_part->setContents(implode("\n", $block['data']));
 
             /* Since private keys should NEVER be sent across email (in fact,
@@ -308,18 +354,20 @@ class IMP_PGP extends Horde_Crypt_pgp {
         }
 
         $text = '';
+        $charset = $part->getCharset();
 
         require_once 'Horde/MIME/Message.php';
 
         $base_ob = &$contents->getBaseObjectPtr();
         $addr = $base_ob->getFromAddress();
 
-        $message = &new MIME_Message();
+        $message = new MIME_Message();
         foreach ($parts as $val) {
+            $val->setCharset($charset);
             $message->addPart($val);
         }
 
-        $mc = &new MIME_Contents($message, array('download' => 'download_attach', 'view' => 'view_attach'), array(&$contents));
+        $mc = new MIME_Contents($message, array('download' => 'download_attach', 'view' => 'view_attach'), array(&$contents));
         $message->buildMessage();
 
         foreach ($message->getParts() as $val) {
@@ -389,12 +437,13 @@ class IMP_PGP extends Horde_Crypt_pgp {
      * Get a public key via a public PGP keyserver.
      *
      * @param string $fingerprint  The fingerprint of the requested key.
+     * @param string $address      The email address of the requested key.
      *
      * @return string  See Horde_Crypt_pgp::getPublicKeyserver()
      */
-    function getFromPublicKeyserver($fingerprint)
+    function getFromPublicKeyserver($fingerprint, $address = null)
     {
-        return $this->_keyserverConnect($fingerprint, 'get');
+        return $this->_keyserverConnect($fingerprint, 'get', $address);
     }
 
     /**
@@ -414,13 +463,14 @@ class IMP_PGP extends Horde_Crypt_pgp {
      *
      * @access private
      *
-     * @param string $data    The data to send to the keyserver.
-     * @param string $method  The method to use - either 'get' or 'put'.
+     * @param string $data        The data to send to the keyserver.
+     * @param string $method      The method to use - either 'get' or 'put'.
+     * @param string $additional  Any additional data.
      *
      * @return string  See Horde_Crypt_pgp::getPublicKeyserver()  -or-
      *                     Horde_Crypt_pgp::putPublicKeyserver().
      */
-    function _keyserverConnect($data, $method)
+    function _keyserverConnect($data, $method, $additional = null)
     {
         global $conf;
 
@@ -428,7 +478,7 @@ class IMP_PGP extends Horde_Crypt_pgp {
             $timeout = (empty($conf['utils']['gnupg_timeout'])) ? PGP_KEYSERVER_TIMEOUT : $conf['utils']['gnupg_timeout'];
             if ($method == 'get') {
                 foreach ($conf['utils']['gnupg_keyserver'] as $server) {
-                    $result = $this->getPublicKeyserver($data, $server, $timeout);
+                    $result = $this->getPublicKeyserver($data, $server, $timeout, $additional);
                     if (!is_a($result, 'PEAR_Error')) {
                         return $result;
                     }
@@ -449,9 +499,8 @@ class IMP_PGP extends Horde_Crypt_pgp {
      * @param string $address    E-mail address of public key.
      * @param string $signature  A PGP signature block.
      *
-     * @return string  See Horde_Crypt_pgp::decryptSignature()
-     *                 -OR-
-     *                 Horde_Crypt_pgp::decryptDetachedSignature()
+     * @return string  See Horde_Crypt_pgp::decryptSignature()  -or-
+     *                     Horde_Crypt_pgp::decryptDetachedSignature().
      */
     function verifySignature($text, $address, $signature = '')
     {
@@ -483,21 +532,30 @@ class IMP_PGP extends Horde_Crypt_pgp {
         return $this->decrypt($text, $options);
     }
 
-
     /**
-     * Decrypt a message with user's public/private keypair.
+     * Decrypt a message with user's public/private keypair or a passphrase.
      *
-     * @param string $text         The text to decrypt.
-     * @param boolean $passphrase  Whether a passphrase has to be used.
+     * @param string $text             The text to decrypt.
+     * @param boolean $symmetric_hint  Whether the text has been encrypted
+     *                                 symmetrically. If null, we try to find
+     *                                 out.
+     * @param boolean $passphrase      Whether a passphrase has to be used.
      *
      * @return string  The decrypted message. Returns PEAR_Error object on
      *                 error.
      */
-    function decryptMessage($text, $passphrase = true)
+    function decryptMessage($text, $symmetric_hint = null, $passphrase = true)
     {
+        if (is_null($symmetric_hint) &&
+            is_callable(array('Horde_Crypt_pgp', 'encryptedSymmetrically'))) {
+            $symmetric_hint = $this->encryptedSymmetrically($text);
+        }
+
         /* decrypt() returns a PEAR_Error object on error. */
         if (!$passphrase) {
             return $this->decrypt($text, array('type' => 'message', 'no_passphrase' => true));
+        } elseif ($symmetric_hint) {
+            return $this->decrypt($text, array('type' => 'message', 'passphrase' => $this->getSymmetricPassphrase()));
         } else {
             return $this->decrypt($text, array('type' => 'message', 'pubkey' => $this->getPersonalPublicKey(), 'privkey' => $this->getPersonalPrivateKey(), 'passphrase' => $this->getPassphrase()));
         }
@@ -510,10 +568,21 @@ class IMP_PGP extends Horde_Crypt_pgp {
      */
     function getPassphrase()
     {
-        global $imp;
+        if (isset($_SESSION['imp']['pgp_passphrase'])) {
+            return Secret::read(Secret::getKey('imp'), $_SESSION['imp']['pgp_passphrase']);
+        }
+    }
 
-        if (isset($imp['pgp_passphrase'])) {
-            return Secret::read(Secret::getKey('imp'), $imp['pgp_passphrase']);
+    /**
+     * Gets the user's passphrase for symmetric encryption from the session
+     * cache.
+     *
+     * @return string  The passphrase, if set.
+     */
+    function getSymmetricPassphrase()
+    {
+        if (isset($_SESSION['imp']['pgp_sym_passphrase'])) {
+            return Secret::read(Secret::getKey('imp'), $_SESSION['imp']['pgp_sym_passphrase']);
         }
     }
 
@@ -530,8 +599,21 @@ class IMP_PGP extends Horde_Crypt_pgp {
             return false;
         }
 
-        $GLOBALS['imp']['pgp_passphrase'] = Secret::write(Secret::getKey('imp'), $passphrase);
+        $_SESSION['imp']['pgp_passphrase'] = Secret::write(Secret::getKey('imp'), $passphrase);
+        return true;
+    }
 
+    /**
+     * Store's the user's passphrase for symmetric encryption in the session
+     * cache.
+     *
+     * @param string $passphrase  The user's passphrase.
+     *
+     * @return boolean  True
+     */
+    function storeSymmetricPassphrase($passphrase)
+    {
+        $_SESSION['imp']['pgp_sym_passphrase'] = Secret::write(Secret::getKey('imp'), $passphrase);
         return true;
     }
 
@@ -540,7 +622,15 @@ class IMP_PGP extends Horde_Crypt_pgp {
      */
     function unsetPassphrase()
     {
-        unset($GLOBALS['imp']['pgp_passphrase']);
+        unset($_SESSION['imp']['pgp_passphrase']);
+    }
+
+    /**
+     * Clear the passphrase for symmetric encryption from the session cache.
+     */
+    function unsetSymmetricPassphrase()
+    {
+        unset($_SESSION['imp']['pgp_sym_passphrase']);
     }
 
     /**
@@ -574,27 +664,21 @@ class IMP_PGP extends Horde_Crypt_pgp {
      *
      * @return string  The javascript link.
      */
-    function getJSOpenWinCode($actionid, $reload = true, $params = null)
+    function getJSOpenWinCode($actionid, $reload = true, $params = array())
     {
-        $popup_url = Horde::applicationUrl('pgp.php');
-        $popup_url = Util::addParameter($popup_url, 'actionID', $actionid, false);
+        $params['actionID'] = $actionid;
         if (!empty($reload)) {
+            require_once 'Horde/SessionObjects.php';
+            $cacheSess = &Horde_SessionObjects::singleton();
+
             if (is_bool($reload)) {
-                $popup_url = Util::addParameter($popup_url, 'reload', Util::removeParameter(Horde::selfUrl(true), array('actionID')), false);
+                $params['reload'] = $cacheSess->storeOid(html_entity_decode(Util::removeParameter(Horde::selfUrl(true), array('actionID'))), false);
             } else {
-                require_once 'Horde/SessionObjects.php';
-                $cacheSess = &Horde_SessionObjects::singleton();
-                $popup_url = Util::addParameter($popup_url, 'passphrase_action', $cacheSess->storeOid($reload, false), false);
+                $params['passphrase_action'] = $cacheSess->storeOid($reload, false);
             }
         }
 
-        if (is_array($params)) {
-            foreach ($params as $key => $val) {
-                $popup_url = Util::addParameter($popup_url, $key, $val, false);
-            }
-        }
-
-        return "popup_imp('" . $popup_url . "',450,200);";
+        return IMP::popupIMPString('pgp.php', $params, 450, 200);
     }
 
     /**
@@ -614,18 +698,23 @@ class IMP_PGP extends Horde_Crypt_pgp {
      *
      * @access private
      *
-     * @param array $addresses  The e-mail address of the keys to use for
-     *                          encryption.
+     * @param array $addresses    The e-mail address of the keys to use for
+     *                            encryption.
+     * @param boolean $symmetric  Whether to encrypt symmetrically.
      *
      * @return array  The list of parameters needed by encrypt().
      *                Returns PEAR_Error on error.
      */
-    function _encryptParameters($addresses)
+    function _encryptParameters($addresses, $symmetric = false)
     {
+        if ($symmetric) {
+            return array('symmetric' => true,
+                         'passphrase' => $this->getSymmetricPassphrase());
+        }
+
         $addr_list = array();
 
         foreach ($addresses as $val) {
-            /* We can only encrypt if we are sending to a single person. */
             $addrOb = IMP::bareAddress($val, true);
             $key_addr = array_pop($addrOb);
 
@@ -663,13 +752,14 @@ class IMP_PGP extends Horde_Crypt_pgp {
      * @param MIME_Part $mime_part  The MIME_Part object to encrypt.
      * @param array $addresses      The e-mail address of the keys to use for
      *                              encryption.
+     * @param boolean $symmetric    Whether to encrypt symmetrically.
      *
      * @return MIME_Part  See Horde_Crypt_pgp::encryptMIMEPart(). Returns
      *                    PEAR_Error object on error.
      */
-    function IMPencryptMIMEPart($mime_part, $addresses)
+    function IMPencryptMIMEPart($mime_part, $addresses, $symmetric = false)
     {
-        $params = $this->_encryptParameters($addresses);
+        $params = $this->_encryptParameters($addresses, $symmetric);
         if (is_a($params, 'PEAR_Error')) {
             return $params;
         }
@@ -682,13 +772,15 @@ class IMP_PGP extends Horde_Crypt_pgp {
      * @param MIME_Part $mime_part  The MIME_Part object to sign and encrypt.
      * @param array $addresses      The e-mail address of the keys to use for
      *                              encryption.
+     * @param boolean $symmetric    Whether to encrypt symmetrically.
      *
      * @return MIME_Part  See Horde_Crypt_pgp::signAndencryptMIMEPart().
      *                    Returns PEAR_Error object on error.
      */
-    function IMPsignAndEncryptMIMEPart($mime_part, $addresses)
+    function IMPsignAndEncryptMIMEPart($mime_part, $addresses,
+                                       $symmetric = false)
     {
-        $encrypt_params = $this->_encryptParameters($addresses);
+        $encrypt_params = $this->_encryptParameters($addresses, $symmetric);
         if (is_a($encrypt_params, 'PEAR_Error')) {
             return $encrypt_params;
         }
