@@ -11,9 +11,9 @@ define('IMP_SMIME_PUBKEY_FIELD', 'smimePublicKey');
  * The IMP_SMIME:: class contains all functions related to handling
  * S/MIME messages within IMP.
  *
- * $Horde: imp/lib/Crypt/SMIME.php,v 1.45.2.22 2008/10/15 02:43:30 slusarz Exp $
+ * $Horde: imp/lib/Crypt/SMIME.php,v 1.45.2.24 2009/02/10 18:47:41 slusarz Exp $
  *
- * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -387,11 +387,12 @@ class IMP_SMIME extends Horde_Crypt_smime {
     {
         $params['actionID'] = $actionid;
         if (!empty($reload)) {
+            require_once 'Horde/SessionObjects.php';
+            $cacheSess = &Horde_SessionObjects::singleton();
+
             if (is_bool($reload)) {
-                $params['reload'] = html_entity_decode(Util::removeParameter(Horde::selfUrl(true), array('actionID')));
+                $params['reload'] = $cacheSess->storeOid(html_entity_decode(Util::removeParameter(Horde::selfUrl(true), array('actionID'))), false);
             } else {
-                require_once 'Horde/SessionObjects.php';
-                $cacheSess = &Horde_SessionObjects::singleton();
                 $params['passphrase_action'] = $cacheSess->storeOid($reload, false);
             }
         }
