@@ -8,7 +8,7 @@ require_once 'Horde/Serialize.php';
  * query information about individual mailboxes.
  * In IMP, folders = IMAP mailboxes so the two terms are used interchangably.
  *
- * $Horde: imp/lib/IMAP/Tree.php,v 1.25.2.67 2009/01/06 15:24:05 jan Exp $
+ * $Horde: imp/lib/IMAP/Tree.php,v 1.25.2.70 2009/05/15 21:40:01 slusarz Exp $
  *
  * Copyright 2000-2009 The Horde Project (http://www.horde.org/)
  *
@@ -32,6 +32,7 @@ define('IMPTREE_ELT_IS_POLLED', 32);
 define('IMPTREE_ELT_NEED_SORT', 64);
 define('IMPTREE_ELT_VFOLDER', 128);
 define('IMPTREE_ELT_NONIMAP', 256);
+define('IMPTREE_ELT_INVISIBLE', 512);
 
 /* The isOpen() expanded mode constants. */
 define('IMPTREE_OPEN_NONE', 0);
@@ -136,6 +137,13 @@ class IMP_Tree {
      */
     var $_subscribed = null;
 
+    /**
+     * The cached full list of mailboxes on the server.
+     *
+     * @var array
+     */
+    var $_fulllist = null;
+
     /**
      * Tree changed flag.  Set when something in the tree has been altered.
      *
@@ -264,10 +272,10 @@ class IMP_Tree {
          * regenerated for each request.
          * Don't store $_currkey, $_currparent, and $_currstack since the
          * user MUST call reset() before cycling through the tree.
-         * Don't store $_subscribed - this information is stored in the
-         * elements.
+         * Don't store $_subscribed and $_fulllist - - this information is
+         * stored in the elements.
          * Reset the $_changed and $_trackdiff flags. */
-        $this->_currkey = $this->_currparent = $this->_eltdiff = $this->_expanded = $this->_imap_sort = $this->_poll = $this->_subscribed = null;
+        $this->_currkey = $this->_currparent = $this->_eltdiff = $this->_expanded = $this->_fulllist = $this->_imap_sort = $this->_poll = $this->_subscribed = null;
         $this->_currstack = array();
         $this->_changed = false;
         $this->_trackdiff = true;
@@ -294,10 +302,8 @@ class IMP_Tree {
      */
     function _getList($showunsub)
     {
-        static $full_list;
-
-        if ($showunsub && isset($full_list)) {
-            return $full_list;
+        if ($showunsub && !is_null($this->_fulllist)) {
+            return $this->_fulllist;
         } elseif (!$showunsub && !is_null($this->_subscribed)) {
             return array_keys($this->_subscribed);
         }
@@ -321,8 +327,8 @@ class IMP_Tree {
 
         // Cached mailbox lists.
         if ($showunsub) {
-            $full_list = array_keys($names);
-            return $full_list;
+            $this->_fulllist = array_keys($names);
+            return $this->_fulllist;
         } else {
             // Need to compare to full list to remove non-existent mailboxes
             // See RFC 3501 [6.3.9]
@@ -404,6 +410,10 @@ class IMP_Tree {
         $elt['l'] = ($elt['v'] == 'INBOX') ? _("Inbox") : String::convertCharset($tmp[$elt['c']], 'UTF7-IMAP');
 
         if ($_SESSION['imp']['base_protocol'] != 'pop3') {
+            if (!empty($GLOBALS['conf']['hooks']['display_folder'])) {
+                $this->_setInvisible($elt, !Horde::callHook('_imp_hook_display_folder', array($elt['v']), 'imp'));
+            }
+
             if ($elt['c'] != 0) {
                 $elt['p'] = implode(is_null($ns_info) ? $this->_delimiter : $ns_info['delimiter'], array_slice($tmp, 0, $elt['c']));
             }
@@ -874,6 +884,13 @@ class IMP_Tree {
 
         $elt = &$this->_tree[$id];
 
+        /* Delete the entry from the folder list cache(s). */
+        foreach (array('_subscribed', '_fulllist') as $var) {
+            if (!is_null($this->$var)) {
+                $this->$var = array_values(array_diff($this->$var, array($id)));
+            }
+        }
+
         /* Do not delete from tree if there are child elements - instead,
          * convert to a container element. */
         if ($this->hasChildren($elt)) {
@@ -1307,6 +1324,34 @@ class IMP_Tree {
         $this->_setAttribute($elt, IMPTREE_ELT_IS_POLLED, $bool);
     }
 
+    /**
+     * Is the element invisible?
+     *
+     * @since IMP 4.3.4
+     *
+     * @param array $elt  A tree element.
+     *
+     * @return integer  True if the element is marked as invisible.
+     */
+    function isInvisible($elt)
+    {
+        return $elt['a'] & IMPTREE_ELT_INVISIBLE;
+    }
+
+    /**
+     * Set the invisible attribute for an element.
+     *
+     * @access private
+     * @since IMP 4.3.4
+     *
+     * @param array &$elt    A tree element.
+     * @param boolean $bool  The setting.
+     */
+    function _setInvisible(&$elt, $bool)
+    {
+        $this->_setAttribute($elt, IMPTREE_ELT_INVISIBLE, $bool);
+    }
+
     /**
      * Flag the element as needing its children to be sorted.
      *
@@ -1477,9 +1522,10 @@ class IMP_Tree {
      */
     function _activeElt($elt)
     {
-        return ($this->_showunsub ||
-                ($this->isSubscribed($elt) && !$this->isContainer($elt)) ||
-                $this->hasChildren($elt));
+        return (!$this->isInvisible($elt) &&
+                ($this->_showunsub ||
+                 ($this->isSubscribed($elt) && !$this->isContainer($elt)) ||
+                 $this->hasChildren($elt)));
     }
 
     /**
