@@ -128,7 +128,7 @@ define('IMP_IDX_SEP', "\1");
 /**
  * IMP Base Class.
  *
- * $Horde: imp/lib/IMP.php,v 1.449.4.124 2009/01/06 15:24:04 jan Exp $
+ * $Horde: imp/lib/IMP.php,v 1.449.4.127 2009/08/05 03:56:37 slusarz Exp $
  *
  * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
@@ -1347,7 +1347,7 @@ class IMP {
      */
     function appendNamespace($mbox)
     {
-        $ns_info = IMP::getNamespace($mbox, false);
+        $ns_info = IMP::getNamespace($mbox);
         if ($ns_info === null) {
             $ns_info = IMP::defaultNamespace();
         }
@@ -1435,11 +1435,13 @@ class IMP {
      *
      * @since IMP 4.2
      *
-     * @param string $default  The default encrypt option.
+     * @param string $default      The default encrypt option.
+     * @param boolean $returnList  Whether to return a hash with options
+     *                             instead of the options tag.
      *
      * @return string  The list of option tags.
      */
-    function encryptList($default = null)
+    function encryptList($default = null, $returnList = false)
     {
         if (empty($default)) {
             $default = $GLOBALS['prefs']->getValue('default_encrypt');
@@ -1462,6 +1464,10 @@ class IMP {
             $enc_options[IMP_SMIME_SIGNENC] = _("S/MIME Sign/Encrypt Message");
         }
 
+        if ($returnList) {
+            return $enc_options;
+        }
+
         $output = '';
         foreach ($enc_options as $key => $val) {
              $output .= '<option value="' . $key . '"' . (($default == $key) ? ' selected="selected"' : '') . '>' . $val . '</option>' . "\n";
@@ -2166,8 +2172,12 @@ class IMP {
         $browser_css = array();
         if ($GLOBALS['browser']->isBrowser('msie')) {
             $ie_major = $GLOBALS['browser']->getMajor();
-            if ($ie_major == 7) {
-                $browser_css[] = 'ie7.css';
+            if ($ie_major >= 7) {
+                if (($ie_major == 7) ||
+                    // IE 8 uses IE 7 compatibility mode for dimp
+                    in_array('dimp', $apps)) {
+                    $browser_css[] = 'ie7.css';
+                }
             } elseif ($ie_major < 7) {
                 $browser_css[] = 'ie6_or_less.css';
                 if ($GLOBALS['browser']->getPlatform() == 'mac') {
